@@ -218,8 +218,8 @@ def interpolate_bad_channels(data, channel_labels=None, h=None, p=1.3, kriging_d
             data[i, :] = 0
             continue
         data[i, :] = np.matmul(weights[imult], data[imult, :])
-    # from easyqc.gui import viewseis
-    # f = viewseis(data.T, si=1/30, h=h, title='interp2', taxis=0)
+    # from viewephys.gui import viewephys
+    # f = viewephys(data.T, fs=1/30, h=h, title='interp2')
     return data
 
 
@@ -335,7 +335,7 @@ def decompress_destripe_cbin(sr_file, output_file=None, h=None, wrot=None, appen
     butter_kwargs, k_kwargs, spatial_fcn = _get_destripe_parameters(sr.fs, butter_kwargs, k_kwargs, k_filter)
     h = sr.geometry if h is None else h
     ncv = h['sample_shift'].size  # number of channels
-    output_file = sr.file_bin.with_suffix('.bin') if output_file is None else output_file
+    output_file = sr.file_bin.with_suffix('.bin') if output_file is None else Path(output_file)
     assert output_file != sr.file_bin
     taper = np.r_[0, scipy.signal.windows.cosine((SAMPLES_TAPER - 1) * 2), 0]
     # create the FFT stencils
@@ -481,8 +481,8 @@ def decompress_destripe_cbin(sr_file, output_file=None, h=None, wrot=None, appen
         rms_data = np.frombuffer(rms_data, dtype=np.float32)
         assert(rms_data.shape[0] == time_data.shape[0] * ncv)
         rms_data = rms_data.reshape(time_data.shape[0], ncv)
-        np.save(Path(sr_file).parent.joinpath('_iblqc_ephysTimeRmsAP.rms.npy'), rms_data)
-        np.save(Path(sr_file).parent.joinpath('_iblqc_ephysTimeRmsAP.timestamps.npy'), time_data)
+        np.save(output_file.parent.joinpath('_iblqc_ephysTimeRmsAP.rms.npy'), rms_data)
+        np.save(output_file.parent.joinpath('_iblqc_ephysTimeRmsAP.timestamps.npy'), time_data)
 
 
 def rcoeff(x, y):
