@@ -22,22 +22,6 @@ one = ONE(**TEST_DB)
 brain_atlas = AllenAtlas(25)
 
 
-class TestProbeInsertion(unittest.TestCase):
-
-    def test_creation(self):
-        probe = [''.join(random.choices(string.ascii_letters, k=5)),
-                 ''.join(random.choices(string.ascii_letters, k=5))]
-        create_alyx_probe_insertions(session_path=EPHYS_SESSION, model='3B2', labels=probe,
-                                     one=one, force=True)
-        insertion = one.alyx.get(f'/insertions?&session={EPHYS_SESSION}', clobber=True)
-        assert(len(insertion) == 2)
-        assert (insertion[0]['json']['qc'] == 'NOT_SET')
-        assert (len(insertion[0]['json']['extended_qc']) == 0)
-
-        one.alyx.rest('insertions', 'delete', id=insertion[0]['id'])
-        one.alyx.rest('insertions', 'delete', id=insertion[1]['id'])
-
-
 class TestTracingQc(unittest.TestCase):
     probe01_id = None
     probe00_id = None
@@ -56,7 +40,7 @@ class TestTracingQc(unittest.TestCase):
 
     def test_tracing_exists(self):
         register_track(self.probe00_id, picks=self.xyz_picks, one=one, overwrite=True,
-                       channels=False)
+                       channels=False, brain_atlas=brain_atlas)
         insertion = one.alyx.get('/insertions/' + self.probe00_id, clobber=True)
 
         assert (insertion['json']['qc'] == 'NOT_SET')
@@ -64,7 +48,7 @@ class TestTracingQc(unittest.TestCase):
 
     def test_tracing_not_exists(self):
         register_track(self.probe01_id, picks=None, one=one, overwrite=True,
-                       channels=False)
+                       channels=False, brain_atlas=brain_atlas)
         insertion = one.alyx.get('/insertions/' + self.probe01_id, clobber=True)
         assert (insertion['json']['qc'] == 'CRITICAL')
         assert (insertion['json']['extended_qc']['tracing_exists'] == 0)
@@ -359,16 +343,6 @@ class TestUploadToFlatIron(unittest.TestCase):
         channels_id = np.load(alf_path.joinpath('channels.brainLocationIds_ccf_2017.npy'))
         assert(channels_mlapdv.shape[0] == channels_id.shape[0])
 
-        clusters_mlapdv = np.load(alf_path.joinpath('clusters.mlapdv.npy'))
-        assert(np.all(np.abs(clusters_mlapdv) > 0))
-        clusters_id = np.load(alf_path.joinpath('clusters.brainLocationIds_ccf_2017.npy'))
-        assert(clusters_mlapdv.shape[0] == clusters_id.shape[0])
-        assert(np.all(np.in1d(clusters_mlapdv, channels_mlapdv)))
-        assert (np.all(np.in1d(clusters_id, channels_id)))
-        clusters_acro = np.load(alf_path.joinpath('clusters.brainLocationAcronyms_ccf_2017.npy'),
-                                allow_pickle=True)
-        assert(clusters_acro.shape == clusters_id.shape)
-
     def test_upload_to_flatiron(self):
         for file in self.file_paths:
             file_registered = one.alyx.get(f'/datasets?&session={EPHYS_SESSION}'
